@@ -1,18 +1,14 @@
-from Products.CMFCore.utils import UniqueObject
 from Products.CMFCore.utils import _checkPermission, \
      _getAuthenticatedUser, limitGrantedRoles
 from Products.CMFCore.utils import getToolByName, _dtmldir
 from Products.CMFCore.WorkflowTool import WorkflowTool as BaseTool
 from Products.CMFPlone import ToolNames
-from AccessControl import getSecurityManager
-from Products.CMFCore.WorkflowCore import WorkflowException
 from ZODB.POSException import ConflictError
 
-from Globals import InitializeClass, DTMLFile
+from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
 from Products.CMFCore import CMFCorePermissions
 from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
-from PloneUtilities import log
 from Products.CMFPlone.PloneBaseTool import PloneBaseTool
 
 class WorkflowTool(PloneBaseTool, BaseTool):
