@@ -12,15 +12,19 @@
 # Folderish objects so that we can suppress content whose
 # id starts with a . - we need a method to do this.
 # Mainly because Portal.py inherients from PortalFolder
-# and not PloneFolder.  But there could many other 
+# and not PloneFolder.  But there could many other
 # instances of 3rd party products that do the same thing.
 # so here is the method.
 
 contents = None
 try:
-    contents = context.listFolderContents(contentFilter=contentFilter, suppressHiddenFiles=suppressHiddenFiles)
+    contents = context.aq_explicit.listFolderContents(contentFilter=contentFilter, suppressHiddenFiles=suppressHiddenFiles)
 except TypeError:
     #XXX Manually do suppression
-    contents = [obj for obj in context.listFolderContents(contentFilter=contentFilter) if obj.getId()[:1]!='.']
+    context.plone_log('Manual fall back in getFolderContents - your Folder.listFolderContents method does not ' \
+                      'support suppressHiddenFiles')
+    contents = [obj
+                for obj in context.aq_explicit.listFolderContents(contentFilter=contentFilter)
+                if not obj.getId().startswith('.')
+               ]
 return contents
-
