@@ -17,11 +17,11 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 """A simple implementation of a Message Catalog.
 
-$Id: GettextMessageCatalog.py,v 1.19 2004/05/04 21:56:03 dreamcatcher Exp $
+$Id: GettextMessageCatalog.py,v 1.24 2004/07/02 13:45:10 fresh Exp $
 """
 
 from gettext import GNUTranslations
-import os, sys, types, codecs, traceback
+import os, sys, types, codecs, traceback, time
 import re
 
 from Acquisition import aq_parent, Implicit
@@ -38,7 +38,11 @@ from OFS.Uninstalled import BrokenClass
 from utils import log, Registry, INFO, BLATHER, PROBLEM
 from msgfmt import Msgfmt
 
-
+try:
+    from Products.PageTemplates.TALES import Context as TALESContextClass
+except ImportError:
+    # must be using OpenTAL
+    TALESContextClass = None
 
 try:
     True
@@ -60,17 +64,6 @@ except ImportError:
             filename = filename + '.pt'
         return PageTemplateFile(filename, '', __name__=id)
 
-# template to use to write missing entries to .missing
-missing_template = u"""msgid "%(id)s"
-msgstr ""
-"""
-
-orig_text_template = u"""
-#. %(text)s
-"""
-
-orig_text_line_joiner = u"\n#. "
-
 permission = 'View management screens'
 
 translationRegistry = Registry()
@@ -236,11 +229,9 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
             self.name = unicode(tro._info.get('language-name', ''), tro._charset)
             self.default_zope_data_encoding = tro._charset
             translationRegistry[self.getId()] = self._v_tro = tro
-            missingFileName = self._pofile[:-2] + '.missing'
+            missingFileName = self._pofile[:-3] + '.missing'
             if os.access(missingFileName, os.W_OK):
-                self._missing = MissingIds(missingFileName, self._v_tro._charset)
-            else:
-                self._missing = None
+                tro.add_fallback(MissingIds(missingFileName, self._v_tro._charset))
             if self.name:
                 self.title = '%s language (%s) for %s' % (self._language, self.name, self._domain)
             else:
@@ -272,25 +263,6 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
             if not REQUEST.form.has_key('noredir'):
                 REQUEST.RESPONSE.redirect(self.absolute_url())
 
-    def _log_missing(self, id, orig_text):
-        """Logging missing ids
-        """
-        if self._missing is None:
-            return
-        self._missing.log(id, orig_text)
-
-    def getMessage(self, id, orig_text=None, testing=False):
-        """get message from catalog
-        """
-        self._prepareTranslations()
-        try:
-            msg = getMessage(self._v_tro, id, orig_text)
-        except KeyError:
-            if not testing:
-                self._log_missing(id, orig_text)
-            raise
-        return msg
-
     security.declarePublic('queryMessage')
     def queryMessage(self, id, default=None):
         """Queries the catalog for a message
@@ -298,7 +270,7 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
         If the message wasn't found the default value or the id is returned.
         """
         try:
-            return self.getMessage(id, default) #, testing=True)
+            return getMessage(translationRegistry[self.getId()],id,default)
         except KeyError:
             if default is None:
                 default = id
@@ -455,7 +427,22 @@ class GettextMessageCatalog(Persistent, Implicit, Traversable, Tabs):
 
 InitializeClass(GettextMessageCatalog)
 
+# template to use to write missing entries to .missing
+missing_template = u'''
+#. Added on %(date)s
+#.
+#: %(reference)s%(context)s
+#.
+#. Orginal text:%(original)s
+msgid "%(id)s"
+msgstr ""
+'''
+
 class MissingIds(Persistent):
+    """This behaves like a gettext message catalog but always
+    raises a KeyError. However, along the way, it adds the msgid that
+    was missing to the .missing file in .po format.
+    """
 
     security = ClassSecurityInfo()
     security.declareObjectProtected(view_management_screens)
@@ -477,15 +464,116 @@ class MissingIds(Persistent):
                 self._ids[msgid] = 1
         file.close()
 
-    def log(self, msgid, orig_text):
+    def gettext(self, msgid):
         if not self._ids.has_key(msgid):
             if getattr(self, '_v_file', None) is None:
                 self._v_file = codecs.open(self._fileName, 'a', self._charset)
-            if orig_text:
-                orig_text = orig_text_line_joiner.join(orig_text.split('\n'))
-                self._v_file.write(orig_text_template % {'text': orig_text})
-            self._v_file.write(missing_template % {'id':msgid.replace('"', r'\"')})
+            context_info = None
+
+            # raise an exception so we can get a stack frame
+            try:
+                raise ZeroDivisionError
+            except ZeroDivisionError:
+                f = sys.exc_info()[2].tb_frame.f_back
+                # gm_f is the frame in which the getMessage in this
+                # file executes. 
+                gm_f = f.f_back
+
+            # The next line provides a way to detect recursion.
+            __exception_formatter__ = 1
+            
+            if TALESContextClass:
+                # with ZPT TALES, we can provide some context as to
+                # where this message was found
+                
+                # go hunting for the context that called up
+                while f is not None:
+                    locals = f.f_locals
+                    if locals.get('__exception_formatter__'):
+                        # Stop recursion.
+                        context_info = ('Recursion Error while trying to find Context',)
+                        break
+                    Context = locals.get('self')
+                    if isinstance(Context,TALESContextClass):
+                        context_info = (
+                            Context.source_file,
+                            'Line %i, Column %i' % (
+                                       Context.position[0],
+                                       Context.position[1],
+                                       )
+                            )
+                        original = f.f_back.f_locals.get('default')
+                        break
+                    f = f.f_back
+
+            if context_info is None:
+                # lets insert details of the thing that called PTS
+                # you can indicate a thing to use instead of PTS
+                # by putting:
+                #
+                # __pts_caller_backcount__ = 1
+                #
+                # ...in the thing.
+                # This is handy, for example, if you have a generic
+                # external method or python script that does
+                # translations and you want to record what is calling
+                # it, and  not just the thing itself.
+                #
+                # The value is the number of stack frames to count
+                # back to find the caller.
+                # For external methods, this should be 2
+                # For python scripts, it might be 1
+                # XXX please add to and correct as appropriate
+
+                # build an inverse list of frames
+                frames = []
+                f = gm_f
+                while 1:
+                    frames.append(f)
+                    f = f.f_back
+                    if f is None:
+                        break
+                frames.reverse()
+                
+                caller_f = None
+                # look for __pts_caller_backcount__
+                for f in frames:
+                    backcount = f.f_locals.get('__pts_caller_backcount__')
+                    if backcount is not None:
+                        while backcount:
+                            f = f.f_back
+                            backcount -= 1
+                        caller_f = f
+                        break
+
+                # delete frames list reference to (hopefuly) stop
+                # memory leaks
+                frames = None
+                
+                # report context info as appropriate
+                if caller_f is None:
+                    context_info = ('No context information could be found',)
+                else:                    
+                    caller_co = caller_f.f_code
+                    context_info = (
+                        caller_co.co_filename,
+                        'Line %i, in %s' % (caller_f.f_lineno, caller_co.co_name),
+                        )
+                    
+            # insert default text, also probably pretty fragile ;-)
+            default = gm_f.f_locals['orig_text']
+            if default is None:
+                '< No original text supplied >'
+                
+            self._v_file.write(missing_template % {
+                'id':msgid.replace('"', r'\"'),
+                'date':time.asctime(),
+                'reference':context_info[0],
+                'context':''.join(['\n#. '+line for line in context_info[1:]]),
+                'original':''.join(['\n#. '+line for line in default.split('\n')])
+                })
             self._v_file.flush()
             self._ids[msgid]=1
+        raise KeyError, msgid
 
 InitializeClass(MissingIds)
