@@ -17,7 +17,7 @@
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 """Placeless Translation Service for providing I18n to file-based code.
 
-$Id: PlacelessTranslationService.py,v 1.33 2004/05/04 21:56:03 dreamcatcher Exp $
+$Id: PlacelessTranslationService.py,v 1.37 2004/07/10 08:52:27 tiran Exp $
 """
 
 import sys, os, re, fnmatch
@@ -108,6 +108,11 @@ class PTSWrapper:
         get the real service and call its translate method
         return default if service couldnt be retrieved
         """
+        
+        # this is useful for GettextMessageCatalog
+        # see the end of GettextMessageCatalog.py for details        
+        __pts_caller_backcount__ = 1
+        
         service = self.load(context)
         if not service: return default
         return service.translate(domain, msgid, mapping, context, target_language, default)
@@ -133,11 +138,16 @@ class PTSWrapper:
         service = self.load(context)
         return service.getLanguages(domain)
 
+    security.declarePrivate('negotiate_language')
+    def negotiate_language(self, context, domain):
+        service = self.load(context)
+        return service.negotiate_language(context.REQUEST,domain)
+
     def __repr__(self):
         """
         return a string representation
         """
-        return "<PTSWrapper for %s>" %(self._path)
+        return "<PTSWrapper for %s>" % '/'.join(self._path)
 
 InitializeClass(PTSWrapper)
 
@@ -153,7 +163,7 @@ class PlacelessTranslationService(Folder):
     # -3 for alpha, -2 for beta, -1 for release candidate
     # for forked releases internal is always 99
     # use an internal of >99 to recreate the PTS at every startup (development mode)
-    _class_version = (1, -1, 15, 99)
+    _class_version = (1, -1, 16, 0)
     all_meta_types = ()
 
     security = ClassSecurityInfo()
